function [ Q_of_theta, theta ] = ComputeObjectiveFunction( sample, varargin )
% [ Q_of_theta, theta ] = ComputeObjectiveFunction( sample, ... )
% 'Weights'
% 'Theta'
% 'Gamma'
%
% Exemple:
% [ Q_of_theta, theta ] = ComputeObjectiveFunction( X ,...
%                         'Weights', w, 'Theta', theta, 'Gamma', K);
%
% Is computed :
% Q(Theta)= \sum_i \log(1 + Weights_i(\frac{sample-Theta}{Gamma})^2)
%

p = inputParser;
addRequired(p, 'sample', @isnumeric);
parse(p, sample);

sample= double(sample(:)');

% default parameters
m= min(sample(:));  M= max(sample(:));
theta= linspace(m - (M-m)/10, M + (M-m)/10, 1000)';
weights= ones(size(sample));
gamma= 1;


addOptional(p, 'Theta', theta,  @(x) ...
    isnumeric(x));
addOptional(p, 'Weights', weights, @(x) ...
    isvector(x) && isnumeric(x) && (length(x(:)) == length(sample(:))));
addOptional(p, 'Gamma', gamma, @isnumeric);

parse(p, sample,varargin{:});


gamma= single(p.Results.Gamma);
weights= single(p.Results.Weights);
theta= single(p.Results.Theta);

weights= weights(:)';
init_size= size(theta);
theta=theta(:);
sample= sample(:)';

Q_of_theta = sum( log( gamma.^ 2 + (( (ones(size(theta))*sample - ...
    theta*ones(size(sample)))).^ 2).*(ones(size(theta))*weights)),2);

Q_of_theta = reshape(Q_of_theta, init_size);
theta = reshape(theta, init_size);
end

