function [out ,gamma] = MyriadFilter(input, gamma,  varargin)
% out = MyriadFilter(input, gamma)
%
% Compute the myriad filtering of the input. 

p = inputParser;
addRequired(p, 'input', @isnumeric);
addRequired(p, 'gamma', @isnumeric);

parse(p, input, gamma);

initialtype = class(input);
input=single(input);

addOptional(p, 'Weights', single(ones(size(input))), @isnumeric);

parse(p, input, gamma, varargin{:});

weights = p.Results.Weights;

weights = single(weights ./ ...
    repmat(sum(weights, 3), [1,1, size(weights,3)]) * size(weights,3));

if isempty(gamma)
    if isequal(weights, single(ones(size(input))))
        [out ,gamma] = MyriadFiltering2_generalized(input,single(ones(size(input))));
    else
        [out ,gamma] = MyriadFiltering2_generalized(input, weights);
    end
else
    if isequal(weights, single(ones(size(input))))
        gamma= single(gamma);
        out = MyriadFiltering(input, gamma );
    else
        gamma= single(gamma);
        out = MyriadFiltering2(input, gamma, weights );
    end
end

    
    
% switch nargin
%     case 1
%         [out ,gamma] = MyriadFilteringGeneralized(input);
%     case 2
%         gamma= single(gamma);
%         out = MyriadFiltering(input, gamma);
%     otherwise
%         error('Error using MyriadFilter: requires 1 or 2 output.');
% end

out = cast(out, initialtype);

end

function [out, varargout] = kCauchyNearestPatches(input, gamma, varargin)
% [CauchySamples] = K_cauchy_best_patches(Img, ...)
% 'SampleSize'
% 'GammaCauchyParameter'
% 'PatchWidth'
% 'SearchWindowRadius'
%
% Example:
% out = K_cauchy_best_patches(input, 'GammaCauchyParameter', 5,...
%                                    'SampleSize', 30,...
%                                    'PatchWidth', 5,...
%                                    'SearchWindowRadius', 10);


% Default param :
size_of_the_sample= single(40);
patch_width=single(3);
search_window_radius=single(15);


p = inputParser;
addRequired(p, 'input', @isnumeric);
addRequired(p, 'gamma', @isnumeric);
addOptional(p, 'SampleSize', size_of_the_sample, @isnumeric);
addOptional(p, 'PatchWidth', patch_width, @isnumeric);
addOptional(p, 'SearchWindowRadius', search_window_radius, @isnumeric);

parse(p, input, gamma, varargin{:});

initialtype = class(input);
input=single(input);
gamma=single(gamma);

size_of_the_sample= single(p.Results.SampleSize);
patch_width= single(p.Results.PatchWidth);
search_window_radius= single(p.Results.SearchWindowRadius);

% out = CauchyPatchExtraction(input, gamma, size_of_the_sample,...
%     patch_width, search_window_radius);

out = cauchy_similarity(input, gamma, size_of_the_sample, patch_width, search_window_radius);


if nargout==2
    [out, poids] = cauchy_similarity_poids(input, gamma, size_of_the_sample, patch_width, search_window_radius);
    out = cast(out, initialtype);
    poids = exp(poids);
    poids= poids./repmat( sum(poids,3), [1,1, size(poids,3)]);
    varargout{1}=cast(poids, 'double');
elseif nargout==1
    out = cauchy_similarity(input, gamma, size_of_the_sample, patch_width, search_window_radius);
    out = cast(out, initialtype);
end


end