%% addCauchyNoise
% Compute a sample image corrupted with a Cauchy noise.
% 
%% Syntax. 
%%
% 
%   [ Xnoisy ] = addCauchyNoise( X, gamma );
%   
%% Definitions. 
% For $\gamma \in R$, and a pixel image X,  the Cauchy distribution is
% given by the following density:
% 
% $$f(t) = \frac{1}{\pi}\; arctan (\frac{ t-X }{\gamma}) + \frac{1 }{2} $$
% 
%% Description.
% |[ Xnoisy ] = addCauchyNoise( X, gamma )| computes a sample image
% corrupted with a Cauchy noise with parameter |gamma|. 
% 
% 
%% Example.

% Define gamma parameter
gamma=5;

% Load an image. 
input= imread('./data/parrot.png');

% Corrupt the image.
noisy = addCauchyNoise( input, gamma);

% Display the objective function.
figure, imshow(noisy), title('Cauchy noise on an image')
