%% k_SSD_NearestPatches
% Extract the centers of the K best patches with L2 norm. 
% 
%% Syntax. 
%%
% 
%   sample = K_SSD_best_patches( X );
%   sample = K_SSD_best_patches( X, ...);
%   
%% Description.
% |sample = kCauchyNearestPatches( X );| returns, for each pixel of the
% image, the centers of the K best patches, in the sense of the L2 norm, in
% a search window around the pixel. 
%% Example.

% Load a gray-scale image and set the gamma parameter. 
input= double(imread('./data/parrot.png'));
input= input + 30*randn(size(input));

sampleFromPatches = K_SSD_best_patches(input, ...
                                   'SampleSize', 10,...
                                   'PatchWidth', 5,...
                                   'SearchWindowRadius', 10);

input=repmat(input, 1, 1, 3);

% Patch extraction. 
sampleFromPatches2 = K_SSD_best_patches(input, ...
                                   'SampleSize', 10,...
                                   'PatchWidth', 5,...
                                   'SearchWindowRadius', 10);

% Display the NL-mean.
imshow(uint8(squeeze(mean(sampleFromPatches,3))))

%% Input arguments.
%
% * |'SampleSize'| is the number of retained patches. Default value is 30.
%
% * |'PatchWidth'| is the width of the patches used for comparison. Default value is 5.
%
% * |'SearchWindowRadius'|  is the radius of search window. Default value is 10.
%