function [out, varargout] = kCauchyNearestPatches(input, gamma, varargin)
% [CauchySamples] = K_cauchy_best_patches(Img, ...)
% 'SampleSize'
% 'GammaCauchyParameter'
% 'PatchWidth'
% 'SearchWindowRadius'
%
% Example:
% out = K_cauchy_best_patches(input, 'GammaCauchyParameter', 5,...
%                                    'SampleSize', 30,...
%                                    'PatchWidth', 5,...
%                                    'SearchWindowRadius', 10);


% Default param :
size_of_the_sample= single(40);
patch_width=single(3);
search_window_radius=single(15);


p = inputParser;
addRequired(p, 'input', @isnumeric);
addRequired(p, 'gamma', @isnumeric);
addOptional(p, 'SampleSize', size_of_the_sample, @isnumeric);
addOptional(p, 'PatchWidth', patch_width, @isnumeric);
addOptional(p, 'SearchWindowRadius', search_window_radius, @isnumeric);

parse(p, input, gamma, varargin{:});

initialtype = class(input);
input=single(input);
gamma=single(gamma);

size_of_the_sample= single(p.Results.SampleSize);
patch_width= single(p.Results.PatchWidth);
search_window_radius= single(p.Results.SearchWindowRadius);

% out = CauchyPatchExtraction(input, gamma, size_of_the_sample,...
%     patch_width, search_window_radius);

out = cauchy_similarity(input, gamma, size_of_the_sample, patch_width, search_window_radius);


if nargout==2
    [out, poids] = cauchy_similarity_poids(input, gamma, size_of_the_sample, patch_width, search_window_radius);
    out = cast(out, initialtype);
    varargout{1}=cast(poids, 'double');
elseif nargout==1
    out = cauchy_similarity(input, gamma, size_of_the_sample, patch_width, search_window_radius);
    out = cast(out, initialtype);
end


end