/* Distance de similarité sous hypothèse de bruit Cauchy */

#ifndef _CRT_SECURE_NO_WARNINGS
#define _CRT_SECURE_NO_WARNINGS
#endif // !_CRT_SECURE_NO_WARNINGS

#include <matrix.h>
#include <mex.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>

#include "MyriadHub.h"

/*
NL_cauchy_similarity.cpp
 
 
 * Linux compiling:
 mex -c Myriad.cpp
 mex -c cauchy_patch_extract.cpp Myriad.o
 mex -c main.cpp cauchy_patch_extract.o
 mex('LDFLAGS=`pkg-config --cflags --libs opencv`', '-largeArrayDims','NL_cauchy_similarity.cpp','main.o')
 
 * Windows compiling
 
 
 */

void mexFunction (int nlhs, mxArray *plhs[],
        int nrhs, const mxArray *prhs[])
{
    
    /* Vérification du nombre d'arguments d'entrée :
     * image input , gamma, K, patch width, search window radius
     **/
    if (nrhs != 5)
    {
        mexErrMsgIdAndTxt("KNNS:nrhs","5 inputs required");
    }
    
    if (nlhs != 1)
    {
        mexErrMsgIdAndTxt("KNNS:nlhs","1 output required");
    }
    
    printf("\n--------------------------\n");
    printf("Call KNN similarity Cauchy\n");
    printf("--------------------------\n");
    
    /*Gestion entree-sortie*/
    
    const size_t * dims = (const size_t*)(mxGetDimensions(prhs[0]));
    int dim = mxGetNumberOfDimensions(prhs[0]);
    
    int wA= (int)dims[0];
    int hA = (int)dims[1];
    
    
    int patch_w = (int)(mxGetScalar(prhs[3]));
    int patch_radius = (patch_w-1)/2;
    float gamma_param= (float)(mxGetScalar(prhs[1]));
    
    int W_radius = (int)(mxGetScalar(prhs[4]));
    int K = (int)(mxGetScalar(prhs[2]));
    
    /* penser à vérifier en entrée que */
    if (((W_radius)*(W_radius))-1 < K)
    {
        mexWarnMsgIdAndTxt("KNNS:checkParam",
        "K is too large in comparison of window radius.\nFrom now K=%d\n",
                W_radius*W_radius-1);
        K=W_radius*W_radius-1;
    }
    
    const int n_dim_result= 3;
    size_t* dims_result = (size_t *)(malloc(sizeof(size_t)*n_dim_result));
    dims_result[0]= (size_t)wA;
    dims_result[1]= (size_t)hA;
    dims_result[2]= (size_t)K+1;
    
    plhs[0]= mxCreateNumericArray(n_dim_result, dims_result ,
            mxSINGLE_CLASS, mxREAL     ) ;
        
    printf("size image:\t %d x %d \ngamma=\t\t %lf \n", wA,hA,gamma_param);
    printf("K= \t\t\t%d \npatch width=\t\t%d\n",  K,  2*patch_radius+1);
    printf("search window width=\t%d\n", 2*W_radius+1  );
    printf("--------------------------\n\n");
    
    
    CauchyPatchExtraction((float*)(mxGetPr(prhs[0])), (float*)(mxGetPr(plhs[0])), (int) wA, (int) hA,
            2*patch_radius+1, 2*W_radius+1 , (int)K , gamma_param );

    
    return;
}
