/*
*	MexMyriadExchange.cpp
*
*	Updated on:	2017-06-26
*	   Authors:	Marcel Brusius,
*				Fabien Pierre
*
*/

#ifndef _CRT_SECURE_NO_WARNINGS
#define _CRT_SECURE_NO_WARNINGS
#endif // !_CRT_SECURE_NO_WARNINGS

#include "MexOpenCVWrapper.h"

using namespace cv;

void Mex2OpenCV( const float *Input , Mat Output )
{
	if( Output.dims == 2 )
	{
		float *OutputPointer;
		for( int i = 0; i < Output.rows; ++i )
		{
			OutputPointer = Output.ptr<float>( i );
			for( int j = 0; j < Output.cols; ++j )
			{
				OutputPointer[ j ] = Input[ j * Output.rows + i ];
			}
		}
	}
	else if( Output.dims == 3 )
	{
		float *OutputPointer = (float *)Output.data;

		for( int i = 0; i < Output.size[ 0 ]; ++i )
		{
			for( int j = 0; j < Output.size[ 1 ]; ++j )
			{
				for( int k = 0; k < Output.size[ 2 ]; ++k )
				{
					OutputPointer[ j + i*Output.size[ 1 ] + k * Output.size[ 1 ] * Output.size[ 0 ] ] = Input[ i + Output.size[ 0 ] * j + Output.size[ 0 ] * Output.size[ 1 ] * k ];
				}
			}
		}
	}
	else
		mexErrMsgTxt( "Too many matrix dimensions. Expected 2 or 3." );
	
	return;
}

void OpenCV2Mex( Mat Input , float *Output )
{
	if( Input.dims == 2 )
	{
		float *InputPointer = (float *)Input.data;

		for( int i = 0; i < Input.rows; ++i )
		{
			for( int j = 0; j < Input.cols; ++j )
			{
                Output[ j * Input.size[ 0 ] + i ] = InputPointer[ j + i * Input.size[ 1 ]];
			}
		}
	}

	else if( Input.dims == 3 )
	{
		float *InputPointer = (float *)Input.data;

		for( int i = 0; i < Input.size[ 0 ]; ++i )
		{
			for( int j = 0; j < Input.size[ 1 ]; ++j )
			{
				for( int k = 0; k < Input.size[ 2 ]; ++k )
				{
					Output[ i + Input.size[ 0 ] * j + Input.size[ 0 ] * Input.size[ 1 ] * k ] 
                            = InputPointer[ j + i*Input.size[ 1 ] + k * Input.size[ 1 ] * Input.size[ 0 ] ];
				}
			}
		}
	}
	else
		mexErrMsgTxt( "Too many matrix dimensions. Expected 2 or 3." );

	return;
}