/*
*	Myriad.h
*
*	Updated on:	2017-06-26
*	   Authors:	Marcel Brusius
*
*/

#pragma once

#ifndef _CRT_SECURE_NO_WARNINGS
#define _CRT_SECURE_NO_WARNINGS
#endif // !_CRT_SECURE_NO_WARNINGS

#include <opencv2/opencv.hpp>
#include <algorithm>
#include <vector>


namespace Filter
{
	class Myriad
	{
	public:
		// constructor, destructor (empty), "=" operator:
		Myriad( cv::Mat inInputImage , int inPatchSize , int inSearchSize , int inRefPatchCount , float ingamma );
		Myriad( cv::Mat inPatchMatrix , float inGamma );
		Myriad( const Myriad& M );
		~Myriad();
		Myriad& operator=( const Myriad& M );


		void PatchExtraction();
		void Compute();
		void ComputeGeneralized();


		void setGamma( float inGamma );
		void setPatchSize( int inPatchSize );
		void setSearchSize( int inSearchSize );
		void setRefPatchCount( int inRefPatchCount );

		cv::Mat getOutputImage();
		cv::Mat getOutputGamma();
		cv::Mat getPatchMatrix();
		void setInputImage( cv::Mat inInputMatrix );

	private:
		int PatchSize;
		int SearchSize;
		int RefPatchCount;

		cv::Mat InputImage;
		cv::Mat PatchMatrix;
		cv::Mat OutputImage;
		cv::Mat OutputGamma;

		float Gamma;

		std::string FilenameInput;
		std::string FilenameOutput;

		cv::Mat FindPatches( int xpos , int ypos );

		float ComputeMyriad( cv::Mat Array );
        void ComputeMyriadGeneralized( cv::Mat Array, float * beta_out, float * gamma_out );

		float ComputeMyriadHelper( cv::Mat Array , float x );

		float SelectionMyriad( cv::Mat Array );
		float SelectionMyriadHelper( cv::Mat Array , float x );

		float Similarity( int Initial_x , int Initial_y , int Reference_x , int Reference_y );
		float Similarity( cv::Mat ImagePatch , cv::Mat ReferencePatch );
		float VecMax( std::vector<float> &sim , std::vector<float> &vec );
	};
}