/* Myriad filtering : wrap with OpenCV code. */

#ifndef _CRT_SECURE_NO_WARNINGS
#define _CRT_SECURE_NO_WARNINGS
#endif // !_CRT_SECURE_NO_WARNINGS

#include <matrix.h>
#include <mex.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>

#include <limits>

//#include "MyriadHub.h"

/*
 * MyriadFiltering.cpp
 */



void MyriadFilterComputation3(float * xi, float * a,
        int wA, int hA, int K, float Gamma, float * wi)
{
    
    float test, tmp,ret, beta;
    float GammaSquare = Gamma*Gamma;
    
    int offset=0;
    int N=wA*hA;
            
    for(offset=0; offset < N; ++offset){
    /*for(int l=0; l < wA; l++){
        ++offset;
        for(int k=0; k < hA; k++){*/
            beta =  std::numeric_limits<float>::infinity();
            
            for( int j = 0; j < K; ++j )
            {
                float x = xi[offset + j*N];
                test = 0.0f;
                for( int i = 0; i < K; ++i )
                {
                    tmp = (xi[offset + i*N] - x);
                    test += wi[offset + i*N] * log(tmp*tmp + GammaSquare);
                    
                }
                
                if( test < beta )
                {
                    beta = test;
                    ret = xi[offset + j*N];
                }
            }
            
            beta = ret;
            float eps = 1.0E-2;
            float dist = std::numeric_limits<float>::infinity();
            float beta_old;
            
            int i=0;
            for(i = 0; i < 10000; ++i )
            {
                beta_old = beta;
                
                float denom = 0, num = 0;
                float tmp;
                for( int j = 0; j < K; ++j )
                {
                    float xe = xi[offset + j*N];
                    float xebeta= xe - beta;
                    tmp = wi[offset + j*N] * 1.0f / (GammaSquare + xebeta*xebeta);
                    num += xe * tmp;
                    denom += tmp;
                }
                
                beta = num / denom;
                
                float signed_dist = beta - beta_old;
                dist =  signed_dist > 0 ? signed_dist : -signed_dist;
                if( dist < eps )
                {
                    break;
                }
            }
            
            //mexPrintf("itt=%d\n", i);
            
            a[offset]= beta;
            
      /*  }
    }*/
    }
    return;
}




void mexFunction (int nlhs, mxArray *plhs[],
        int nrhs, const mxArray *prhs[])
{
    
    /* Check input numbers :
     * image input , gamma, K, patch width, search window radius
     **/
    if (nrhs != 3)
    {
        mexErrMsgIdAndTxt("KNNS:nrhs","3 inputs required");
    }
    
    if (nlhs != 1)
    {
        mexErrMsgIdAndTxt("KNNS:nlhs","1 output required");
    }
    
    mexPrintf("\n--------------------------\n");
    mexPrintf("Call Myriad filter\n");
    mexPrintf("--------------------------\n");
    
    /*input-output*/
    
    const size_t * dims = (const size_t*)(mxGetDimensions(prhs[0]));
    int dim = (int)mxGetNumberOfDimensions(prhs[0]);
    
    int wA= (int)dims[0];
    int hA = (int)dims[1];
    int K = (int)dims[2];
    
    float gamma_param= (float)(mxGetScalar(prhs[1]));
    
    
    const int n_dim_result= 2;
    size_t* dims_result = (size_t *)(malloc(sizeof(size_t)*n_dim_result));
    dims_result[0]= (size_t)wA;
    dims_result[1]= (size_t)hA;
    
    plhs[0]= mxCreateNumericArray(n_dim_result, dims_result ,
            mxSINGLE_CLASS, mxREAL     ) ;
    
    mexPrintf("size image:\t %d x %d \ngamma=\t\t %lf \n", wA,hA,gamma_param);
    mexPrintf("--------------------------\n\n");
    
    /* MyriadFilterComputation((float*)(mxGetPr(prhs[0])),
     * (float*)(mxGetPr(plhs[0])), (int) wA, (int) hA,
     * (int) K, gamma_param);*/
    
    float * left= (float *)(malloc(sizeof(float)*wA*hA));
    float * right= (float *)(malloc(sizeof(float)*wA*hA*K));
    
    int N =wA*hA*K;
    float * entree =(float*)(mxGetPr(prhs[0]));
    for(int ind = 0; ind < N; ++ind){
        right[ind] = entree[ind];
    }
    
    MyriadFilterComputation3(right, left, (int) wA, (int) hA,
            (int) K, gamma_param, (float*)(mxGetPr(prhs[2])));
    
    N =wA*hA;
    float * resultat = (float*)(mxGetPr(plhs[0]));
    for(int ind = 0; ind < N; ++ind){
        resultat[ind] = left[ind];
    }
    return;
}
