/* Myriad filtering : wrap with OpenCV code. */

#ifndef _CRT_SECURE_NO_WARNINGS
#define _CRT_SECURE_NO_WARNINGS
#endif // !_CRT_SECURE_NO_WARNINGS

#include <matrix.h>
#include <mex.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>

#include <limits>

//#include "MyriadHub.h"

/*
 * MyriadFiltering.cpp
 */



void MyriadFilterComputationGeneralized(float * xi, float * a, float * gamma_out, float * wi,
        int wA, int hA, int K, float gamma_init)
{
    int N=wA*hA;
    float inv_K = 1.0f / K;
    float ret;
    float GammaSquare = gamma_init*gamma_init;
            
    for(int offset=0; offset < N; ++offset){
        
        // represents the main fixed point iteration:
        /* init of beta*/
        
            float   beta =  std::numeric_limits<float>::infinity();
            
            for( int j = 0; j < K; ++j )
            {
                float x = xi[offset + j*N];
                float test = 0.0f;
                for( int i = 0; i < K; ++i )
                {
                    float tmp = (xi[offset + i*N] - x);
                    test += wi[offset + i*N] * log(tmp*tmp + GammaSquare);
                    
                }
                
                if( test < beta )
                {
                    beta = test;
                    ret = xi[offset + j*N];
                }
            }
            
            beta = ret;
        //float beta= xi[offset + 0*N];
        
        /* END of init of beta*/
        
        float gamma = 10.0f; // default initialization value.
        float eps = 1.0E-2;
        
        for( int i = 0; i < 1000; ++i ){
            
            float beta_old = beta;
            float gamma_old = gamma;
            
            float S0r = 0.0f;
            float S1r = 0.0f;
            
            for( int j = 0; j < K; ++j ){
                float pre_calc = (xi[offset + j*N]-beta)/gamma;
                float pre_calc2 = pre_calc*pre_calc;
                S0r += wi[offset + j*N]/(1+pre_calc2);
                S1r += wi[offset + j*N]*pre_calc/(1+pre_calc2);
               /* S0r += 1.0f/(1+pre_calc2);
                S1r += pre_calc/(1+pre_calc2); */
            }
            S0r *= inv_K;
            S1r *= inv_K;
           
            beta += gamma * S1r / S0r ;
            gamma = sqrt(gamma*gamma *(1/S0r - 1));
            
            float signed_dist = beta - beta_old;
            float dist1 =  signed_dist > 0 ? signed_dist : -signed_dist;
            signed_dist = gamma - gamma_old;
            float dist2 =  signed_dist > 0 ? signed_dist : -signed_dist;
            
            if( dist1 + dist2 < eps )
            {
                break;
            }
             
        }
        
       a[offset]=beta;
       gamma_out[offset]=gamma;
        
    }
    return;
}





void mexFunction (int nlhs, mxArray *plhs[],
        int nrhs, const mxArray *prhs[])
{
    
    /* Check input numbers :
     * image input , gamma, K, patch width, search window radius
     **/
    if (nrhs != 2)
    {
        mexErrMsgIdAndTxt("KNNS:nrhs","2 inputs required");/*sample, weights*/
    }
    
    if (nlhs != 2)
    {
        mexErrMsgIdAndTxt("KNNS:nlhs","2 output required");/*argmin, gamma*/
    }
    
    mexPrintf("\n------------------------------\n");
    mexPrintf("Call Generalized Myriad filter\n");
    mexPrintf("------------------------------\n");
    
    /*input-output*/
    
    const size_t * dims = (const size_t*)(mxGetDimensions(prhs[0]));
    int dim = (int)mxGetNumberOfDimensions(prhs[0]);
    
    int wA= (int)dims[0];
    int hA = (int)dims[1];
    int K = (int)dims[2];
    
    
    const int n_dim_result= 2;
    size_t* dims_result = (size_t *)(malloc(sizeof(size_t)*n_dim_result));
    dims_result[0]= (size_t)wA;
    dims_result[1]= (size_t)hA;
    
    plhs[0]= mxCreateNumericArray(n_dim_result, dims_result ,
            mxSINGLE_CLASS, mxREAL     ) ;
    plhs[1]= mxCreateNumericArray(n_dim_result, dims_result ,
            mxSINGLE_CLASS, mxREAL     ) ;
    
    mexPrintf("size image:\t %d x %d \n", wA,hA);
    mexPrintf("--------------------------\n\n");
    
    MyriadFilterComputationGeneralized((float*)(mxGetPr(prhs[0])),
            (float*)(mxGetPr(plhs[0])),  (float*)(mxGetPr(plhs[1])), 
            (float*)(mxGetPr(prhs[1])), (int) wA, (int) hA,
            (int) K, 10.0f);
    
    return;
}
