/* Myriad filtering : wrap with OpenCV code. */

#ifndef _CRT_SECURE_NO_WARNINGS
#define _CRT_SECURE_NO_WARNINGS
#endif // !_CRT_SECURE_NO_WARNINGS

#include <matrix.h>
#include <mex.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>

#include "MyriadHub.h"

/*
MyriadFiltering.cpp
*/


void mexFunction (int nlhs, mxArray *plhs[],
        int nrhs, const mxArray *prhs[])
{
    
    /* Check input numbers :
     * image input , gamma, K, patch width, search window radius
     **/
    if (nrhs != 1)
    {
        mexErrMsgIdAndTxt("KNNS:nrhs","1 inputs required");
    }
    
    if (nlhs != 2)
    {
        mexErrMsgIdAndTxt("KNNS:nlhs","2 output required");
    }
    
    mexPrintf("\n------------------------------\n");
    mexPrintf("Call Generalized Myriad filter\n");
    mexPrintf("------------------------------\n");
    
    /*input-output*/
    
    const size_t * dims = (const size_t*)(mxGetDimensions(prhs[0]));
    int dim = (int)mxGetNumberOfDimensions(prhs[0]);
    
    int wA= (int)dims[0];
    int hA = (int)dims[1];
    int K = (int)dims[2];
    
    
    const int n_dim_result= 2;
    size_t* dims_result = (size_t *)(malloc(sizeof(size_t)*n_dim_result));
    dims_result[0]= (size_t)wA;
    dims_result[1]= (size_t)hA;
    
    plhs[0]= mxCreateNumericArray(n_dim_result, dims_result ,
            mxSINGLE_CLASS, mxREAL     ) ;
    plhs[1]= mxCreateNumericArray(n_dim_result, dims_result ,
            mxSINGLE_CLASS, mxREAL     ) ;
        
    mexPrintf("size image:\t %d x %d \n", wA,hA);
    mexPrintf("--------------------------\n\n");
    
    MyriadFilterComputationGeneralized((float*)(mxGetPr(prhs[0])),
            (float*)(mxGetPr(plhs[0])), (float*)(mxGetPr(plhs[1])), (int) wA, (int) hA,
            (int) K, 22.0f);
    
    return;
}
