/*
*	CauchyPatch.cpp
*
*	Updated on:	2017-06-26
*	   Authors:	Marcel Brusius,
*				Fabien Pierre
*
*/

#include "MyriadHub.h"

using namespace cv;
using namespace Filter;

void CauchyPatchExtraction( const float *Input , float *Output , int Height , int Width , int PatchSize , int SearchWindowSize , int NumberReferencePatches , float Gamma )
{
	Mat InputImage( Height , Width , CV_32F );
	Mex2OpenCV( Input , InputImage );

	Myriad M( InputImage , PatchSize , SearchWindowSize , NumberReferencePatches , Gamma );

	M.PatchExtraction();
	Mat Patches = M.getPatchMatrix();

	OpenCV2Mex( Patches , Output );

	return;
}

void MyriadFilterComputation( const float *Input , float *Output , int Height , int Width, int NumberReferencePatches , float Gamma )
{
	int dims[] = { Height, Width, NumberReferencePatches };
	Mat PatchMatrix( 3 , dims , CV_32F );
	Mex2OpenCV( Input , PatchMatrix );

	Myriad M( PatchMatrix , Gamma );

	M.Compute();

	Mat Result = M.getOutputImage();

	OpenCV2Mex( Result , Output );

	return;
}

void MyriadFilterComputationGeneralized( const float *Input , float *Output , float *OutputGamma , int Height , int Width, int NumberReferencePatches , float Gamma )
{
	int dims[] = { Height, Width, NumberReferencePatches };
	Mat PatchMatrix( 3 , dims , CV_32F );
	Mex2OpenCV( Input , PatchMatrix );

	Myriad M( PatchMatrix , Gamma );

	M.ComputeGeneralized();

	Mat Result = M.getOutputImage();
	Mat ResultGamma = M.getOutputGamma();

	OpenCV2Mex( Result , Output );
	OpenCV2Mex( ResultGamma , OutputGamma );

	return;
}
