#include <string>
#include <ctime>

#include "Myriad.h"
#include "MexMyriadExchange.h"
#include "CauchyPatch.h"

int main( int argc , char** argv )
{
	std::string Filename = argv[ 1 ]; // TODO: filename should not be neccessary in the future
	int PatchSize = atoi(argv[ 2 ]);
	int SearchSize = atoi(argv[ 3 ]);
	int RefCount = atoi(argv[ 4 ]);
	float Gamma = atof( argv[ 5 ] );
	
	std::cout << "*******************************************************************************" << std::endl;
	std::cout << "********************************               ********************************" << std::endl;
	std::cout << "********************************     Input     ********************************" << std::endl;
	std::cout << "********************************               ********************************" << std::endl;
	std::cout << "*******************************************************************************" << std::endl;
	std::cout << std::endl;
	std::cout << "Gamma \t\t\t: " << Gamma << std::endl;
	std::cout << "Window size \t\t: " << PatchSize << "x" << PatchSize << std::endl;
	std::cout << "Search window \t\t: " << SearchSize << "x" << SearchSize << std::endl;
	std::cout << "Reference patches \t: " << RefCount << std::endl;
	std::cout << std::endl;
	std::cout << "*******************************************************************************" << std::endl;
	std::cout << std::endl;
	std::cout << "Press [Enter] to start computation...";
	std::cin.ignore();
	std::cout << std::endl;
	std::cout << "Computation started ..." << std::endl;
	std::cout << std::endl;
	std::cout << std::endl;

	cv::Mat InputImage;
	cv::imread( Filename ).convertTo( InputImage , CV_32F );
	//InputImage /= 255.0f;

	size_t start = clock();
	Filter::Myriad M = Filter::Myriad( InputImage , PatchSize , SearchSize , RefCount , Gamma );
	M.Patch();
	size_t step = clock();

	std::cout << "*******************************************************************************" << std::endl;
	std::cout << "********************************               ********************************" << std::endl;
	std::cout << "******************************** Patch Search  ********************************" << std::endl;
	std::cout << "********************************               ********************************" << std::endl;
	std::cout << "*******************************************************************************" << std::endl;
	std::cout << std::endl;
	std::cout << "Elapsed time is : \t" << float( step - start ) / CLOCKS_PER_SEC << std::endl;
	std::cout << std::endl;

	step = clock();
	M.Compute();
	size_t stop = clock();

	cv::imwrite( "Out.png", M.getOutputImage() );

	std::cout << "*******************************************************************************" << std::endl;
	std::cout << "********************************               ********************************" << std::endl;
	std::cout << "******************************** Myriad Filter ********************************" << std::endl;
	std::cout << "********************************               ********************************" << std::endl;
	std::cout << "*******************************************************************************" << std::endl;
	std::cout << std::endl;
	std::cout << "Elapsed time is : \t" << float( stop - step ) / CLOCKS_PER_SEC << std::endl;
	std::cout << std::endl;

	std::cout << "*******************************************************************************" << std::endl;
	std::cout << "********************************               ********************************" << std::endl;
	std::cout << "********************************    Result     ********************************" << std::endl;
	std::cout << "********************************               ********************************" << std::endl;
	std::cout << "*******************************************************************************" << std::endl;
	std::cout << std::endl;
	std::cout << "Total elapsed time is : \t" << float( stop - start ) / CLOCKS_PER_SEC << std::endl;
	std::cout << std::endl;
	std::cout << "*******************************************************************************" << std::endl;
	std::cout << std::endl;
	std::cout << "Press [Enter] to continue... ";
	std::cin.ignore();

	return 0;
}